package org.apache.bcel.generic;

import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import org.apache.bcel.classfile.AnnotationElementValue;
import org.apache.bcel.classfile.AnnotationEntry;
import org.apache.bcel.classfile.ArrayElementValue;
import org.apache.bcel.classfile.ClassElementValue;
import org.apache.bcel.classfile.ElementValue;
import org.apache.bcel.classfile.EnumElementValue;
import org.apache.bcel.classfile.SimpleElementValue;

public abstract class ElementValueGen {
   public static final int STRING = 115;
   public static final int ENUM_CONSTANT = 101;
   public static final int CLASS = 99;
   public static final int ANNOTATION = 64;
   public static final int ARRAY = 91;
   public static final int PRIMITIVE_INT = 73;
   public static final int PRIMITIVE_BYTE = 66;
   public static final int PRIMITIVE_CHAR = 67;
   public static final int PRIMITIVE_DOUBLE = 68;
   public static final int PRIMITIVE_FLOAT = 70;
   public static final int PRIMITIVE_LONG = 74;
   public static final int PRIMITIVE_SHORT = 83;
   public static final int PRIMITIVE_BOOLEAN = 90;
   /** @deprecated */
   @Deprecated
   protected int type;
   /** @deprecated */
   @Deprecated
   protected ConstantPoolGen cpGen;

   public static ElementValueGen copy(ElementValue value, ConstantPoolGen cpool, boolean copyPoolEntries) {
      switch (value.getElementValueType()) {
         case 64:
            return new AnnotationElementValueGen((AnnotationElementValue)value, cpool, copyPoolEntries);
         case 65:
         case 69:
         case 71:
         case 72:
         case 75:
         case 76:
         case 77:
         case 78:
         case 79:
         case 80:
         case 81:
         case 82:
         case 84:
         case 85:
         case 86:
         case 87:
         case 88:
         case 89:
         case 92:
         case 93:
         case 94:
         case 95:
         case 96:
         case 97:
         case 98:
         case 100:
         case 102:
         case 103:
         case 104:
         case 105:
         case 106:
         case 107:
         case 108:
         case 109:
         case 110:
         case 111:
         case 112:
         case 113:
         case 114:
         default:
            throw new UnsupportedOperationException("Not implemented yet! (" + value.getElementValueType() + ")");
         case 66:
         case 67:
         case 68:
         case 70:
         case 73:
         case 74:
         case 83:
         case 90:
         case 115:
            return new SimpleElementValueGen((SimpleElementValue)value, cpool, copyPoolEntries);
         case 91:
            return new ArrayElementValueGen((ArrayElementValue)value, cpool, copyPoolEntries);
         case 99:
            return new ClassElementValueGen((ClassElementValue)value, cpool, copyPoolEntries);
         case 101:
            return new EnumElementValueGen((EnumElementValue)value, cpool, copyPoolEntries);
      }
   }

   public static ElementValueGen readElementValue(DataInput dis, ConstantPoolGen cpGen) throws IOException {
      int type = dis.readUnsignedByte();
      switch (type) {
         case 64:
            return new AnnotationElementValueGen(64, new AnnotationEntryGen(AnnotationEntry.read(dis, cpGen.getConstantPool(), true), cpGen, false), cpGen);
         case 65:
         case 69:
         case 71:
         case 72:
         case 75:
         case 76:
         case 77:
         case 78:
         case 79:
         case 80:
         case 81:
         case 82:
         case 84:
         case 85:
         case 86:
         case 87:
         case 88:
         case 89:
         case 92:
         case 93:
         case 94:
         case 95:
         case 96:
         case 97:
         case 98:
         case 100:
         case 102:
         case 103:
         case 104:
         case 105:
         case 106:
         case 107:
         case 108:
         case 109:
         case 110:
         case 111:
         case 112:
         case 113:
         case 114:
         default:
            throw new IllegalArgumentException("Unexpected element value kind in annotation: " + type);
         case 66:
            return new SimpleElementValueGen(66, dis.readUnsignedShort(), cpGen);
         case 67:
            return new SimpleElementValueGen(67, dis.readUnsignedShort(), cpGen);
         case 68:
            return new SimpleElementValueGen(68, dis.readUnsignedShort(), cpGen);
         case 70:
            return new SimpleElementValueGen(70, dis.readUnsignedShort(), cpGen);
         case 73:
            return new SimpleElementValueGen(73, dis.readUnsignedShort(), cpGen);
         case 74:
            return new SimpleElementValueGen(74, dis.readUnsignedShort(), cpGen);
         case 83:
            return new SimpleElementValueGen(83, dis.readUnsignedShort(), cpGen);
         case 90:
            return new SimpleElementValueGen(90, dis.readUnsignedShort(), cpGen);
         case 91:
            int numArrayVals = dis.readUnsignedShort();
            ElementValue[] evalues = new ElementValue[numArrayVals];

            for(int j = 0; j < numArrayVals; ++j) {
               evalues[j] = ElementValue.readElementValue(dis, cpGen.getConstantPool());
            }

            return new ArrayElementValueGen(91, evalues, cpGen);
         case 99:
            return new ClassElementValueGen(dis.readUnsignedShort(), cpGen);
         case 101:
            return new EnumElementValueGen(dis.readUnsignedShort(), dis.readUnsignedShort(), cpGen);
         case 115:
            return new SimpleElementValueGen(115, dis.readUnsignedShort(), cpGen);
      }
   }

   protected ElementValueGen(int type, ConstantPoolGen cpGen) {
      this.type = type;
      this.cpGen = cpGen;
   }

   public abstract void dump(DataOutputStream var1) throws IOException;

   protected ConstantPoolGen getConstantPool() {
      return this.cpGen;
   }

   public abstract ElementValue getElementValue();

   public int getElementValueType() {
      return this.type;
   }

   public abstract String stringifyValue();
}
